@@ -27,16 +27,13 @@
 
 #include "libwhoopsie-preferences.h"
 
-#define CONFIG_PATH "/etc/default/whoopsie"
+#define CONFIG_PATH "/etc/whoopsie"
 
 static GMainLoop* loop = NULL;
 static guint loop_shutdown = 0;
 static GKeyFile* key_file = NULL;
 static PolkitAuthority* authority = NULL;
 
-/* Once upstart has an interface for disabiling jobs via initctl, we wont need
- * a configuration file anymore */
-
 gboolean
 whoopsie_preferences_load_configuration (void)
 {
@@ -50,7 +47,6 @@ whoopsie_preferences_load_configuration (void)
         return TRUE;
     }
 
-    g_key_file_set_boolean (key_file, "General", "report_crashes", TRUE);
     g_key_file_set_boolean (key_file, "General", "report_metrics", TRUE);
     data = g_key_file_to_data (key_file, &data_size, &error);
     if (error) {
@@ -72,6 +68,36 @@ out:
     return ret;
 }
 
+static gboolean
+whoopsie_preferences_whoopsie_enabled (GError **error) {
+    char* systemd_check_command[] = { "/bin/systemctl", "-q", "is-enabled", "whoopsie.service", NULL };
+    gchar* upstart_override_content = NULL;
+    gboolean report_crashes = FALSE;
+    int exit_code = 0;
+
+    /* systemd running */
+    if (g_file_test ("/run/systemd/system", G_FILE_TEST_IS_DIR)) {
+        if (g_spawn_sync (NULL, systemd_check_command, NULL, 0, NULL, NULL, NULL, NULL, &exit_code, error))
+            report_crashes = (exit_code == 0);
+    }
+    /* upstart running */
+    else {
+        report_crashes = TRUE;
+        if (!g_file_get_contents("/etc/init/whoopsie.override", &upstart_override_content, NULL, error)) {
+            /* Only warn if the file exist but we couldn't access */
+            if (g_error_matches (*error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
+                g_clear_error (error);
+        }
+        else {
+           if (g_strrstr(upstart_override_content, "manual") != NULL)
+               report_crashes = FALSE;
+        }
+        g_free(upstart_override_content);
+    }
+
+    return report_crashes;
+}
+
 static void
 whoopsie_preferences_dbus_notify (WhoopsiePreferences* interface)
 {
@@ -80,22 +106,21 @@ whoopsie_preferences_dbus_notify (WhoopsiePreferences* interface)
     gboolean report_metrics = FALSE;
     gboolean automatically_report_crashes = FALSE;
 
-    report_crashes = g_key_file_get_boolean (key_file, "General",
-                                             "report_crashes", &error);
+    report_crashes = whoopsie_preferences_whoopsie_enabled (&error);
     if (error) {
-        g_warning ("Could not get crashes: %s", error->message);
+        g_warning ("Could not get if whoopsie is enabled: %s", error->message);
         g_error_free (error);
+        error = NULL;
     }
 
-    error = NULL;
     report_metrics = g_key_file_get_boolean (key_file, "General",
                                              "report_metrics", &error);
     if (error) {
         g_warning ("Could not get metrics: %s", error->message);
         g_error_free (error);
     }
-    automatically_report_crashes = access ("/var/lib/apport/autoreport",
-                                           F_OK) != -1;
+    automatically_report_crashes = g_file_test ("/var/lib/apport/autoreport",
+                                                G_FILE_TEST_EXISTS);
     whoopsie_preferences_set_report_crashes (interface, report_crashes);
     whoopsie_preferences_set_automatically_report_crashes (
             interface,
@@ -119,7 +144,7 @@ gboolean
 autoreport_file_changed (GFileMonitor* monitor, GFile* file, GFile* other_file, GFileMonitorEvent event_type, gpointer user_data)
 {
     WhoopsiePreferences* interface = user_data;
-    if (access ("/var/lib/apport/autoreport", F_OK) != -1) {
+    if (g_file_test ("/var/lib/apport/autoreport", G_FILE_TEST_EXISTS)) {
         whoopsie_preferences_set_automatically_report_crashes (interface, TRUE);
     } else {
         whoopsie_preferences_set_automatically_report_crashes (interface, FALSE);
@@ -169,13 +194,12 @@ whoopsie_preferences_load (WhoopsiePreferences* interface)
 }
 
 void
-whoopsie_preferences_notify_upstart (gboolean run_whoopsie)
+whoopsie_preferences_notify_init (gboolean run_whoopsie)
 {
     GError* error = NULL;
-    char* command[] = { "/sbin/initctl", run_whoopsie ? "start" : "stop", "whoopsie", NULL };
-    if (!g_spawn_sync (NULL, command, NULL, 0, NULL, NULL, NULL, NULL, NULL, &error)) {
-        g_print ("Could not run initctl: %s\n", error->message);
-    }
+    char* command[] = { "/usr/sbin/invoke-rc.d", "whoopsie", run_whoopsie ? "start" : "stop", NULL };
+    if (!g_spawn_sync (NULL, command, NULL, 0, NULL, NULL, NULL, NULL, NULL, &error))
+        g_print ("Could not run invoke-rc.d: %s\n", error->message);
 }
 
 void
@@ -198,12 +222,13 @@ whoopsie_automatic_reporting_changed (WhoopsiePreferences* object,
      * writing the file when it already exists. */
     if (iface->get_automatically_report_crashes (object)) {
         if ((g_mkdir_with_parents("/var/lib/apport/", 0755) == 0) &&
-                (access ("/var/lib/apport/autoreport", F_OK) < 0)) {
-            fclose (fopen ("/var/lib/apport/autoreport", "w"));
+                !g_file_test ("/var/lib/apport/autoreport",
+                              G_FILE_TEST_EXISTS)) {
+            fclose (g_fopen ("/var/lib/apport/autoreport", "w"));
         }
     } else {
-        if (access ("/var/lib/apport/autoreport", F_OK) != -1) {
-            unlink ("/var/lib/apport/autoreport");
+        if (g_file_test ("/var/lib/apport/autoreport", G_FILE_TEST_EXISTS)) {
+            g_unlink ("/var/lib/apport/autoreport");
         }
     }
 
@@ -220,36 +245,50 @@ whoopsie_preferences_changed (WhoopsiePreferences* object, GParamSpec* pspec,
     char* data;
     gsize data_size;
     gboolean crashes = !g_strcmp0(user_data, "report_crashes");
+    char* crash_enable_command[] = { "/usr/sbin/update-rc.d", "whoopsie", NULL, NULL };
 
     increase_shutdown_timeout ();
 
     iface = WHOOPSIE_PREFERENCES_GET_IFACE (object);
-    saved_value = g_key_file_get_boolean (key_file, "General", user_data,
-                                            &error);
+    if (crashes) {
+        saved_value = whoopsie_preferences_whoopsie_enabled (&error);
+        new_value = iface->get_report_crashes (object);
+        crash_enable_command[2] = new_value ? "enable" : "disable";
+    } else {
+        new_value = iface->get_report_metrics (object);
+        saved_value = g_key_file_get_boolean (key_file, "General", user_data,
+                                              &error);
+    }
     if (error) {
         g_print ("Could not process configuration: %s\n", error->message);
     }
-    if (crashes)
-        new_value = iface->get_report_crashes (object);
-    else
-        new_value = iface->get_report_metrics (object);
 
     if (error || (saved_value != new_value)) {
         if (error)
             g_error_free (error);
-        g_key_file_set_boolean (key_file, "General", user_data, new_value);
-        data = g_key_file_to_data (key_file, &data_size, &error);
-        if (error) {
-            g_print ("Could not process configuration: %s\n", error->message);
-            return FALSE;
-        }
-        if (!g_file_set_contents (CONFIG_PATH, data, data_size, &error)) {
-            g_print ("Could not write configuration: %s\n", error->message);
-            return FALSE;
+        error = NULL;
+
+        if (crashes) {
+            if (!g_spawn_sync (NULL, crash_enable_command, NULL, 0, NULL, NULL, NULL, NULL, NULL, &error)) {
+                g_print ("Could not run invoke-rc.d: %s\n", error->message);
+                g_error_free (error);
+                return FALSE;
+            }
+        } else {
+            g_key_file_set_boolean (key_file, "General", user_data, new_value);
+            data = g_key_file_to_data (key_file, &data_size, &error);
+            if (error) {
+                g_print ("Could not process configuration: %s\n", error->message);
+                return FALSE;
+            }
+            if (!g_file_set_contents (CONFIG_PATH, data, data_size, &error)) {
+                g_print ("Could not write configuration: %s\n", error->message);
+                return FALSE;
+            }
         }
     }
     if (crashes)
-        whoopsie_preferences_notify_upstart (new_value);
+        whoopsie_preferences_notify_init (new_value);
 
     return TRUE;
 }
